package lt.marsrutai.mobile.serialize;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import lt.marsrutai.mobile.android.dao.dto.ArrivalTimeDto;
import lt.marsrutai.mobile.android.dao.dto.BusStopDto;
import lt.marsrutai.mobile.android.dao.dto.BusTrackDto;
import lt.marsrutai.mobile.android.dao.dto.DaysOfWeekDto;
import lt.marsrutai.mobile.android.dao.dto.ScheduleDto;
import lt.marsrutai.mobile.android.dao.dto.ScopeInfoDto;
import lt.marsrutai.mobile.android.dao.dto.SearchResultDto;
import lt.marsrutai.serialize.exception.SerializeException;

import com.dyuproject.protostuff.LinkedBuffer;
import com.dyuproject.protostuff.ProtostuffIOUtil;
import com.dyuproject.protostuff.Schema;
import com.dyuproject.protostuff.runtime.RuntimeSchema;

public class ProtostuffSerializer implements Serializer {

    @SuppressWarnings("rawtypes")
    private static final Map<String, Schema> schemaMap = new HashMap<String, Schema>();

    private static final Map<String, Object> objectMap = new HashMap<String, Object>();

    static {
        schemaMap.put("getAvailableMobileScopes", RuntimeSchema.getSchema(ScopeInfoDto.class));
        schemaMap.put("getStopNames", RuntimeSchema.getSchema(String.class));
        schemaMap.put("getStops", RuntimeSchema.getSchema(BusStopDto.class));
        schemaMap.put("getStopsNear", RuntimeSchema.getSchema(BusStopDto.class));
        schemaMap.put("getStopById", RuntimeSchema.getSchema(BusStopDto.class));
        schemaMap.put("getArrivalTimes", RuntimeSchema.getSchema(ArrivalTimeDto.class));
        schemaMap.put("getSchedules", RuntimeSchema.getSchema(ScheduleDto.class));
        schemaMap.put("getScheduleById", RuntimeSchema.getSchema(ScheduleDto.class));
        schemaMap.put("getTracks", RuntimeSchema.getSchema(BusTrackDto.class));
        schemaMap.put("getTrackById", RuntimeSchema.getSchema(BusTrackDto.class));
        schemaMap.put("getTrackStops", RuntimeSchema.getSchema(BusStopDto.class));
        schemaMap.put("getDifferentRunningDays", RuntimeSchema.getSchema(DaysOfWeekDto.class));
        schemaMap.put("getStopTimes", RuntimeSchema.getSchema(ArrivalTimeDto.class));
        schemaMap.put("findRoutes", RuntimeSchema.getSchema(SearchResultDto.class));

        objectMap.put("getStopById", null);
        objectMap.put("getScheduleById", null);
        objectMap.put("getTrackById", null);
        objectMap.put("findRoutes", null);
    }

    @Override
    public byte[] writeObject(SerializeRequest request) {

        if (!schemaMap.containsKey(request.getName())) {
            throw new SerializeException("Unknown object name: " + request.getName());
        }

        List<Object> objectList = null;
        if (objectMap.containsKey(request.getName())) {
            objectList = new ArrayList<Object>();
            objectList.add(request.getObject());
        }
        else {
            objectList = (List<Object>) request.getObject();
        }

        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            GZIPOutputStream gos = new GZIPOutputStream(bos);

            ProtostuffIOUtil.writeListTo(gos, objectList, schemaMap.get(request.getName()), LinkedBuffer.allocate(256));

            gos.flush();
            gos.close();
            return bos.toByteArray();
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

    @Override
    public Object readObject(DeserializeRequest request) {

        if (!schemaMap.containsKey(request.getName())) {
            throw new SerializeException("Unknown object name: " + request.getName());
        }

        try {
            ByteArrayInputStream bis = new ByteArrayInputStream(request.getArray());
            GZIPInputStream gis = new GZIPInputStream(bis);

            Object object = ProtostuffIOUtil.parseListFrom(gis, schemaMap.get(request.getName()));

            if (objectMap.containsKey(request.getName())) {
                List<Object> objectList = (List<Object>) object;
                if (null != objectList && objectList.size() == 1) {
                    object = objectList.get(0);
                }
            }

            return object;
        }
        catch (IOException e) {
            throw new SerializeException(e);
        }
    }

}
